package javax.swing.event;

import sun.reflect.misc.ReflectUtil;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.lang.reflect.Array;
import java.util.EventListener;
import java.io.Serializable;

public class EventListenerList implements Serializable
{
    private static final Object[] NULL_ARRAY;
    protected transient Object[] listenerList;
    
    public EventListenerList() {
        this.listenerList = EventListenerList.NULL_ARRAY;
    }
    
    public Object[] getListenerList() {
        return this.listenerList;
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> componentType) {
        Object[] listenerList = this.listenerList;
        EventListener[] array = (EventListener[])(EventListener[])Array.newInstance(componentType, this.getListenerCount(listenerList, componentType));
        int n = 0;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == componentType) {
                array[n++] = (EventListener)listenerList[i + 1];
            }
        }
        return array;
    }
    
    public int getListenerCount() {
        return this.listenerList.length / 2;
    }
    
    public int getListenerCount(Class<?> clazz) {
        return this.getListenerCount(this.listenerList, clazz);
    }
    
    private int getListenerCount(Object[] array, Class clazz) {
        int n = 0;
        for (int i = 0; i < array.length; i += 2) {
            if (clazz == (Class)array[i]) {
                ++n;
            }
        }
        return n;
    }
    
    public synchronized <T extends EventListener> void add(Class<T> obj, T obj2) {
        if (obj2 == null) {
            return;
        }
        if (!obj.isInstance(obj2)) {
            throw new IllegalArgumentException("Listener " + obj2 + " is not of type " + obj);
        }
        if (this.listenerList == EventListenerList.NULL_ARRAY) {
            this.listenerList = new Object[] { obj, obj2 };
        }
        else {
            int length = this.listenerList.length;
            Object[] listenerList = new Object[length + 2];
            System.arraycopy(this.listenerList, 0, listenerList, 0, length);
            listenerList[length] = obj;
            listenerList[length + 1] = obj2;
            this.listenerList = listenerList;
        }
        return;
    }
    
    public synchronized <T extends EventListener> void remove(Class<T> obj, T t) {
        if (t == null) {
            return;
        }
        if (!obj.isInstance(t)) {
            throw new IllegalArgumentException("Listener " + t + " is not of type " + obj);
        }
        int n = -1;
        int i = this.listenerList.length - 2;
        while (i >= 0) {
            if (this.listenerList[i] == obj && this.listenerList[i + 1].equals(t)) {
                n = i;
                break;
            }
            i -= 2;
        }
        if (n != -1) {
            Object[] array = new Object[this.listenerList.length - 2];
            System.arraycopy(this.listenerList, 0, array, 0, n);
            if (n < array.length) {
                System.arraycopy(this.listenerList, n + 2, array, n, array.length - n);
            }
            this.listenerList = ((array.length == 0) ? EventListenerList.NULL_ARRAY : array);
        }
        return;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Object[] listenerList = this.listenerList;
        objectOutputStream.defaultWriteObject();
        for (int i = 0; i < listenerList.length; i += 2) {
            Class<Object> clazz = (Class)listenerList[i];
            EventListener obj = (EventListener)listenerList[i + 1];
            if (obj != null && obj instanceof Serializable) {
                objectOutputStream.writeObject(clazz.getName());
                objectOutputStream.writeObject(obj);
            }
        }
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.listenerList = EventListenerList.NULL_ARRAY;
        objectInputStream.defaultReadObject();
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
            EventListener eventListener = (Object)objectInputStream.readObject();
            String s = (String)object;
            ReflectUtil.checkPackageAccess(s);
            this.add(Class.forName(s, true, contextClassLoader), eventListener);
        }
    }
    
    @Override
    public String toString() {
        Object[] listenerList = this.listenerList;
        String str = "EventListenerList: " + listenerList.length / 2 + " listeners: ";
        for (int i = 0; i <= listenerList.length - 2; i += 2) {
            str = str + " type " + ((Class)listenerList[i]).getName() + " listener " + listenerList[i + 1];
        }
        return str;
    }
    
    static {
        NULL_ARRAY = new Object[0];
    }
}
